@@ -26,24 +26,22 @@
 # noinspection PyUnresolvedReferences
 import bpy
 # noinspection PyUnresolvedReferences
+import bgl
+# noinspection PyUnresolvedReferences
 import blf
-from blf import ROTATION
 from math import fabs, degrees, radians, sqrt, cos, sin, pi
 from mathutils import Vector, Matrix
 from bmesh import from_edit_mesh
 from bpy_extras import view3d_utils, mesh_utils
 import bpy_extras.object_utils as object_utils
 from sys import exc_info
-# GPU
-import bgl
-import gpu
-from gpu_extras.batch import batch_for_shader
 
-shader = gpu.shader.from_builtin('2D_UNIFORM_COLOR') if not bpy.app.background else None
 
 # -------------------------------------------------------------
 # Draw segments
 #
+# rgb: Color
+# fsize: Font size
 # -------------------------------------------------------------
 # noinspection PyUnresolvedReferences,PyUnboundLocalVariable
 def draw_segments(context, myobj, op, region, rv3d):
@@ -56,38 +54,28 @@ def draw_segments(context, myobj, op, region, rv3d):
         ovr = scene.measureit_ovr
         ovrcolor = scene.measureit_ovr_color
         ovrfsize = scene.measureit_ovr_font
-        ovrfang = get_angle_in_rad(scene.measureit_ovr_font_rotation)
-        ovrfaln = scene.measureit_ovr_font_align
         ovrline = scene.measureit_ovr_width
         units = scene.measureit_units
-        fang = get_angle_in_rad(scene.measureit_font_rotation)
-
         # --------------------
         # Scene Scale
         # --------------------
         if scene.measureit_scale is True:
             prs = scene.measureit_scale_precision
             fmts = "%1." + str(prs) + "f"
-            pos_2d = get_scale_txt_location(context)
+            pos_x, pos_y = get_scale_txt_location(context)
             tx_dsp = fmts % scene.measureit_scale_factor
             tx_scale = scene.measureit_gl_scaletxt + " 1:" + tx_dsp
-            draw_text(myobj, pos_2d,
-                      tx_scale, scene.measureit_scale_color, scene.measureit_scale_font,
-                      text_rot=fang)
-
+            draw_text(myobj, pos_x, pos_y,
+                      tx_scale, scene.measureit_scale_color, scene.measureit_scale_font)
         # --------------------
         # Loop
         # --------------------
-        for idx in range(op.measureit_num):
+        for idx in range(0, op.measureit_num):
             ms = op.measureit_segments[idx]
             if ovr is False:
                 fsize = ms.glfont_size
-                fang = get_angle_in_rad(ms.glfont_rotat)
-                faln = ms.glfont_align
             else:
                 fsize = ovrfsize
-                fang = ovrfang
-                faln = ovrfaln
             # ------------------------------
             # only active and visible
             # ------------------------------
@@ -99,10 +87,9 @@ def draw_segments(context, myobj, op, region, rv3d):
                 # noinspection PyBroadException
                 try:
                     if ovr is False:
-                        rgba = ms.glcolor
+                        rgb = ms.glcolor
                     else:
-                        rgba = ovrcolor
-
+                        rgb = ovrcolor
                     # ----------------------
                     # Segment or Label
                     # ----------------------
@@ -112,7 +99,6 @@ def draw_segments(context, myobj, op, region, rv3d):
                         if ms.glpointa <= len(obverts) and ms.glpointb <= len(obverts):
                             a_p1 = get_point(obverts[ms.glpointa].co, myobj)
                             b_p1 = get_point(obverts[ms.glpointb].co, myobj)
-
                     # ----------------------
                     # Segment or Label
                     # ----------------------
@@ -253,8 +239,8 @@ def draw_segments(context, myobj, op, region, rv3d):
                             loc = get_location(myobj)
                             midpoint3d = interpolate3d(a_p1, b_p1, fabs(dist / 2))
                             vn = Vector((midpoint3d[0] - loc[0],
-                                         midpoint3d[1] - loc[1],
-                                         midpoint3d[2] - loc[2]))
+                                                   midpoint3d[1] - loc[1],
+                                                   midpoint3d[2] - loc[2]))
                     else:
                         vn = Vector((ms.glnormalx, ms.glnormaly, ms.glnormalz))
 
@@ -322,7 +308,6 @@ def draw_segments(context, myobj, op, region, rv3d):
                     screen_point_v22 = get_2d_point(region, rv3d, v22)
                     screen_point_v11a = get_2d_point(region, rv3d, v11a)
                     screen_point_v11b = get_2d_point(region, rv3d, v11b)
-
                     # ------------------------------------
                     # colour + line setup
                     # ------------------------------------
@@ -331,6 +316,8 @@ def draw_segments(context, myobj, op, region, rv3d):
                     else:
                         bgl.glLineWidth(ovrline)
 
+                    bgl.glColor4f(rgb[0], rgb[1], rgb[2], rgb[3])
+
                     # ------------------------------------
                     # Text (distance)
                     # ------------------------------------
@@ -340,10 +327,7 @@ def draw_segments(context, myobj, op, region, rv3d):
                         try:
                             midpoint3d = interpolate3d(v1, v2, fabs(dist / 2))
                             gap3d = (midpoint3d[0], midpoint3d[1], midpoint3d[2] + s / 2)
-                            tmp_point = get_2d_point(region, rv3d, gap3d)
-                            if tmp_point is None:
-                                pass
-                            txtpoint2d = tmp_point[0] + ms.glfontx, tmp_point[1] + ms.glfonty
+                            txtpoint2d = get_2d_point(region, rv3d, gap3d)
                             # Scale
                             if scene.measureit_scale is True:
                                 dist = dist * scene.measureit_scale_factor
@@ -369,7 +353,8 @@ def draw_segments(context, myobj, op, region, rv3d):
                             if scene.measureit_gl_show_n is True and ms.glnames is True:
                                 msg += ms.gltxt
                             if scene.measureit_gl_show_d is True or scene.measureit_gl_show_n is True:
-                                draw_text(myobj, txtpoint2d, msg, rgba, fsize, faln, fang)
+                                draw_text(myobj, txtpoint2d[0] + ms.glfontx, txtpoint2d[1] + ms.glfonty,
+                                          msg, rgb, fsize)
 
                             # ------------------------------
                             # if axis loc, show a indicator
@@ -384,7 +369,7 @@ def draw_segments(context, myobj, op, region, rv3d):
                                 if ms.glocz is True:
                                     txt += "Z"
                                 txt += "]"
-                                draw_text(myobj, txtpoint2d, txt, rgba, fsize - 1, text_rot=fang)
+                                draw_text(myobj, txtpoint2d[0], txtpoint2d[1], txt, rgb, fsize - 1)
 
                         except:
                             pass
@@ -393,13 +378,16 @@ def draw_segments(context, myobj, op, region, rv3d):
                     # ------------------------------------
                     # noinspection PyBroadException
                     if ms.gltype == 2 or ms.gltype == 9 or ms.gltype == 11:
+                        right = False
                         tx_dist = ""
                         # noinspection PyBroadException
                         try:
                             if ms.gltype == 2:
                                 tx_dist = ms.gltxt
+                                right = False
                             if ms.gltype == 9:  # Angles
                                 ang = ang_1.angle(ang_2)
+                                right = True
                                 if bpy.context.scene.unit_settings.system_rotation == "DEGREES":
                                     ang = degrees(ang)
 
@@ -411,6 +399,7 @@ def draw_segments(context, myobj, op, region, rv3d):
                                 if scene.measureit_gl_show_n is True:
                                     tx_dist += " " + ms.gltxt
                             if ms.gltype == 11:  # arc
+                                right = True
                                 # print length or arc and angle
                                 if ms.glarc_len is True:
                                     tx_dist = ms.glarc_txlen + format_distance(fmt, units, arc_length)
@@ -432,26 +421,23 @@ def draw_segments(context, myobj, op, region, rv3d):
                                     msg = tx_dist + " "
                                 else:
                                     msg = " "
-
                                 if scene.measureit_gl_show_n is True and ms.glnames is True:
                                     msg += ms.gltxt
 
                                 if scene.measureit_gl_show_d is True or scene.measureit_gl_show_n is True:
                                     # Normal vector
                                     vna = Vector((b_p1[0] - a_p1[0],
-                                                  b_p1[1] - a_p1[1],
-                                                  b_p1[2] - a_p1[2]))
+                                                            b_p1[1] - a_p1[1],
+                                                            b_p1[2] - a_p1[2]))
                                     vna.normalize()
                                     via = vna * ms.glspace
 
                                     gap3d = (b_p1[0] + via[0], b_p1[1] + via[1], b_p1[2] + via[2])
-                                    tmp_point = get_2d_point(region, rv3d, gap3d)
-                                    if tmp_point is not None:
-                                        txtpoint2d = tmp_point[0] + ms.glfontx, tmp_point[1] + ms.glfonty
-                                        draw_text(myobj, txtpoint2d, msg, rgba, fsize, faln, fang)
+                                    txtpoint2d = get_2d_point(region, rv3d, gap3d)
+                                    draw_text(myobj, txtpoint2d[0] + ms.glfontx, txtpoint2d[1] + ms.glfonty, msg, rgb,
+                                              fsize, right)
                                 # Radius
-                                if scene.measureit_gl_show_d is True and ms.gldist is True and \
-                                        ms.glarc_rad is True:
+                                if scene.measureit_gl_show_d is True and ms.gldist is True and ms.glarc_rad is True:
                                     tx_dist = ms.glarc_txradio + format_distance(fmt, units,
                                                                                  dist * scene.measureit_scale_factor)
                                 else:
@@ -461,10 +447,9 @@ def draw_segments(context, myobj, op, region, rv3d):
                             else:
                                 gap3d = (a_p1[0], a_p1[1], a_p1[2])
 
-                            tmp_point = get_2d_point(region, rv3d, gap3d)
-                            if tmp_point is not None:
-                                txtpoint2d = tmp_point[0] + ms.glfontx, tmp_point[1] + ms.glfonty
-                                draw_text(myobj, txtpoint2d, tx_dist, rgba, fsize, faln, fang)
+                            txtpoint2d = get_2d_point(region, rv3d, gap3d)
+                            draw_text(myobj, txtpoint2d[0] + ms.glfontx, txtpoint2d[1] + ms.glfonty, tx_dist, rgb,
+                                      fsize, right)
                         except:
                             pass
                     # ------------------------------------
@@ -473,35 +458,37 @@ def draw_segments(context, myobj, op, region, rv3d):
                     # noinspection PyBroadException
                     if ms.gltype == 10:
                         # noinspection PyBroadException
-                        tx_dist = ms.gltxt
-                        gap3d = (vn1[0], vn1[1], vn1[2])
-                        tmp_point = get_2d_point(region, rv3d, gap3d)
-                        if tmp_point is not None:
-                            txtpoint2d = tmp_point[0] + ms.glfontx, tmp_point[1] + ms.glfonty
-                            draw_text(myobj, txtpoint2d, tx_dist, rgba, fsize, faln, fang)
-
+                        try:
+                            tx_dist = ms.gltxt
+                            gap3d = (vn1[0], vn1[1], vn1[2])
+                            txtpoint2d = get_2d_point(region, rv3d, gap3d)
+                            draw_text(myobj, txtpoint2d[0] + ms.glfontx, txtpoint2d[1] + ms.glfonty,
+                                      tx_dist, rgb, fsize)
+                        except:
+                            pass
                     # ------------------------------------
                     # Draw lines
                     # ------------------------------------
                     bgl.glEnable(bgl.GL_BLEND)
+                    bgl.glColor4f(rgb[0], rgb[1], rgb[2], rgb[3])
 
                     if ms.gltype == 1:  # Segment
-                        draw_line(screen_point_ap1, screen_point_v11, rgba)
-                        draw_line(screen_point_bp1, screen_point_v22, rgba)
-                        draw_arrow(screen_point_v1, screen_point_v2, rgba, a_size, a_type, b_type)
+                        draw_line(screen_point_ap1, screen_point_v11)
+                        draw_line(screen_point_bp1, screen_point_v22)
+                        draw_arrow(screen_point_v1, screen_point_v2, a_size, a_type, b_type)
 
                     if ms.gltype == 12 or ms.gltype == 13 or ms.gltype == 14:  # Segment to origin
-                        draw_line(screen_point_ap1, screen_point_v11, rgba)
-                        draw_line(screen_point_bp1, screen_point_v22, rgba)
-                        draw_arrow(screen_point_v1, screen_point_v2, rgba, a_size, a_type, b_type)
+                        draw_line(screen_point_ap1, screen_point_v11)
+                        draw_line(screen_point_bp1, screen_point_v22)
+                        draw_arrow(screen_point_v1, screen_point_v2, a_size, a_type, b_type)
 
                     if ms.gltype == 2:  # Label
-                        draw_line(screen_point_v11a, screen_point_v11b, rgba)
-                        draw_arrow(screen_point_ap1, screen_point_v11, rgba, a_size, a_type, b_type)
+                        draw_line(screen_point_v11a, screen_point_v11b)
+                        draw_arrow(screen_point_ap1, screen_point_v11, a_size, a_type, b_type)
 
                     if ms.gltype == 3 or ms.gltype == 4 or ms.gltype == 5 or ms.gltype == 8 \
                             or ms.gltype == 6 or ms.gltype == 7:  # Origin and Links
-                        draw_arrow(screen_point_ap1, screen_point_bp1, rgba, a_size, a_type, b_type)
+                        draw_arrow(screen_point_ap1, screen_point_bp1, a_size, a_type, b_type)
 
                     if ms.gltype == 9:  # Angle
                         dist, distloc = distance(an_p1, an_p2)
@@ -514,25 +501,25 @@ def draw_segments(context, myobj, op, region, rv3d):
                         screen_point_an_p2 = get_2d_point(region, rv3d, an_p2)
                         screen_point_an_p3 = get_2d_point(region, rv3d, mp2)
 
-                        draw_line(screen_point_an_p1, screen_point_an_p2, rgba)
-                        draw_line(screen_point_an_p2, screen_point_an_p3, rgba)
-                        draw_line(screen_point_an_p1, screen_point_an_p3, rgba)
+                        draw_line(screen_point_an_p1, screen_point_an_p2)
+                        draw_line(screen_point_an_p2, screen_point_an_p3)
+                        draw_line(screen_point_an_p1, screen_point_an_p3)
 
                     if ms.gltype == 11:  # arc
                         # draw line from center of arc second point
                         c = Vector(a_p1)
                         if ms.glarc_rad is True:
                             if ms.glarc_extrad is False:
-                                draw_arrow(screen_point_ap1, screen_point_bp1, rgba, a_size, a_type, b_type)
+                                draw_arrow(screen_point_ap1, screen_point_bp1, a_size, a_type, b_type)
                             else:
                                 vne = Vector((b_p1[0] - a_p1[0],
-                                              b_p1[1] - a_p1[1],
-                                              b_p1[2] - a_p1[2]))
+                                                        b_p1[1] - a_p1[1],
+                                                        b_p1[2] - a_p1[2]))
                                 vne.normalize()
                                 vie = vne * ms.glspace
                                 pe = (b_p1[0] + vie[0], b_p1[1] + vie[1], b_p1[2] + vie[2])
                                 screen_point_pe = get_2d_point(region, rv3d, pe)
-                                draw_arrow(screen_point_ap1, screen_point_pe, rgba, a_size, a_type, b_type)
+                                draw_arrow(screen_point_ap1, screen_point_pe, a_size, a_type, b_type)
 
                         # create arc around the centerpoint
                         # rotation matrix around normal vector at center point
@@ -543,14 +530,14 @@ def draw_segments(context, myobj, op, region, rv3d):
                             step = arc_angle / n_step
                         else:
                             step = radians(360.0) / n_step
-
+                        #
                         mat_rot1 = Matrix.Rotation(step, 4, vn)
                         mat_trans2 = Matrix.Translation(c)
                         p1 = Vector(an_p1)  # first point of arc
                         # Normal vector
                         vn = Vector((p1[0] - a_p1[0],
-                                     p1[1] - a_p1[1],
-                                     p1[2] - a_p1[2]))
+                                               p1[1] - a_p1[1],
+                                               p1[2] - a_p1[2]))
                         vn.normalize()
                         vi = vn * ms.glspace
 
@@ -559,8 +546,8 @@ def draw_segments(context, myobj, op, region, rv3d):
                         p_02a = None
                         p_02b = None
                         # draw the arc
-                        for i in range(int(n_step)):
-                            p2 = mat_trans2 @ mat_rot1 @ mat_trans1 @ p1
+                        for i in range(0, int(n_step)):
+                            p2 = mat_trans2 * mat_rot1 * mat_trans1 * p1
                             p1_ = (p1[0] + vi[0], p1[1] + vi[1], p1[2] + vi[2])
                             # First Point
                             if i == 0:
@@ -569,8 +556,8 @@ def draw_segments(context, myobj, op, region, rv3d):
 
                             # Normal vector
                             vn = Vector((p2[0] - a_p1[0],
-                                         p2[1] - a_p1[1],
-                                         p2[2] - a_p1[2]))
+                                                   p2[1] - a_p1[1],
+                                                   p2[2] - a_p1[2]))
                             vn.normalize()
                             vi = vn * ms.glspace
 
@@ -579,11 +566,11 @@ def draw_segments(context, myobj, op, region, rv3d):
                             screen_point_p1 = get_2d_point(region, rv3d, p1_)
                             screen_point_p2 = get_2d_point(region, rv3d, p2_)
                             if i == 0:
-                                draw_arrow(screen_point_p1, screen_point_p2, rgba, ms.glarc_s, ms.glarc_a, "99")
+                                draw_arrow(screen_point_p1, screen_point_p2, ms.glarc_s, ms.glarc_a, "99")
                             elif i == int(n_step) - 1:
-                                draw_arrow(screen_point_p1, screen_point_p2, rgba, ms.glarc_s, "99", ms.glarc_b)
+                                draw_arrow(screen_point_p1, screen_point_p2, ms.glarc_s, "99", ms.glarc_b)
                             else:
-                                draw_line(screen_point_p1, screen_point_p2, rgba)
+                                draw_line(screen_point_p1, screen_point_p2)
 
                             p1 = p2.copy()
 
@@ -599,8 +586,8 @@ def draw_segments(context, myobj, op, region, rv3d):
                             screen_point_p2a = get_2d_point(region, rv3d, p_02a)
                             screen_point_p2b = get_2d_point(region, rv3d, p_02b)
 
-                            draw_line(screen_point_p1a, screen_point_p1b, rgba)
-                            draw_line(screen_point_p2a, screen_point_p2b, rgba)
+                            draw_line(screen_point_p1a, screen_point_p1b)
+                            draw_line(screen_point_p2a, screen_point_p2b)
 
                     if ms.gltype == 20:  # Area
                         obverts = get_mesh_vertices(myobj)
@@ -608,9 +595,9 @@ def draw_segments(context, myobj, op, region, rv3d):
                         for face in ms.measureit_faces:
                             myvertices = []
                             for v in face.measureit_index:
-                                myvertices.append(v.glidx)
+                                myvertices.extend([v.glidx])
 
-                            area = get_area_and_paint(myvertices, myobj, obverts, region, rv3d, rgba)
+                            area = get_area_and_paint(myvertices, myobj, obverts, region, rv3d)
                             tot += area
                         # Draw Area number over first face
                         if len(ms.measureit_faces) > 0:
@@ -623,6 +610,7 @@ def draw_segments(context, myobj, op, region, rv3d):
 
                             d1, dn = distance(p1, p2)
                             midpoint3d = interpolate3d(p1, p2, fabs(d1 / 2))
+                            txtpoint2d = get_2d_point(region, rv3d, midpoint3d)
                             # Scale
                             if scene.measureit_scale is True:
                                 tot = tot * scene.measureit_scale_factor
@@ -640,11 +628,9 @@ def draw_segments(context, myobj, op, region, rv3d):
                             if scene.measureit_gl_show_n is True and ms.glnames is True:
                                 msg += ms.gltxt
                             if scene.measureit_gl_show_d is True or scene.measureit_gl_show_n is True:
-                                tmp_point = get_2d_point(region, rv3d, midpoint3d)
-                                if tmp_point is not None:
-                                    txtpoint2d = tmp_point[0] + ms.glfontx, tmp_point[1] + ms.glfonty
-                                    # todo: swap ms.glcolorarea with ms.glcolor ?
-                                    draw_text(myobj, txtpoint2d, msg, ms.glcolorarea, fsize, faln, fang)
+                                draw_text(myobj, txtpoint2d[0] + ms.glfontx, txtpoint2d[1] + ms.glfonty,
+                                          msg, ms.glcolorarea,
+                                          fsize)
 
                 except IndexError:
                     ms.glfree = True
@@ -659,7 +645,7 @@ def draw_segments(context, myobj, op, region, rv3d):
 # Get polygon area and paint area
 #
 # ------------------------------------------
-def get_area_and_paint(myvertices, myobj, obverts, region, rv3d, rgba):
+def get_area_and_paint(myvertices, myobj, obverts, region, rv3d):
     mymesh = myobj.data
     totarea = 0
     if len(myvertices) > 3:
@@ -670,7 +656,7 @@ def get_area_and_paint(myvertices, myobj, obverts, region, rv3d, rgba):
             bm = from_edit_mesh(myobj.data)
             myv = []
             for v in bm.verts:
-                myv.append(v.co)
+                myv.extend([v.co])
             tris = mesh_utils.ngon_tessellate(myv, myvertices)
 
         for t in tris:
@@ -682,11 +668,9 @@ def get_area_and_paint(myvertices, myobj, obverts, region, rv3d, rgba):
             screen_point_p1 = get_2d_point(region, rv3d, p1)
             screen_point_p2 = get_2d_point(region, rv3d, p2)
             screen_point_p3 = get_2d_point(region, rv3d, p3)
-
-            draw_triangle(screen_point_p1, screen_point_p2, screen_point_p3, rgba)
+            draw_triangle(screen_point_p1, screen_point_p2, screen_point_p3)
 
             # Area
-
             area = get_triangle_area(p1, p2, p3)
 
             totarea += area
@@ -699,7 +683,7 @@ def get_area_and_paint(myvertices, myobj, obverts, region, rv3d, rgba):
         screen_point_p1 = get_2d_point(region, rv3d, p1)
         screen_point_p2 = get_2d_point(region, rv3d, p2)
         screen_point_p3 = get_2d_point(region, rv3d, p3)
-        draw_triangle(screen_point_p1, screen_point_p2, screen_point_p3, rgba)
+        draw_triangle(screen_point_p1, screen_point_p2, screen_point_p3)
 
         # Area
         area = get_triangle_area(p1, p2, p3)
@@ -754,7 +738,7 @@ def get_group_sum(myobj, tag):
         scale = bpy.context.scene.unit_settings.scale_length
         tot = 0.0
         obverts = get_mesh_vertices(myobj)
-        for idx in range(mp.measureit_num):
+        for idx in range(0, mp.measureit_num):
             ms = mp.measureit_segments[idx]
             if (ms.gltype == 1 or ms.gltype == 12 or
                 ms.gltype == 13 or ms.gltype == 14) and ms.gltot != '99' \
@@ -801,17 +785,12 @@ def get_group_sum(myobj, tag):
 # -------------------------------------------------------------
 # Create OpenGL text
 #
+# right: Align to right
 # -------------------------------------------------------------
-def draw_text(myobj, pos2d, display_text, rgba, fsize, align='L', text_rot=0.0):
-    if pos2d is None:
-        return
-
-    # dpi = bpy.context.preferences.system.dpi
+def draw_text(myobj, x_pos, y_pos, display_text, rgb, fsize, right=False):
     gap = 12
-    x_pos, y_pos = pos2d
     font_id = 0
     blf.size(font_id, fsize, 72)
-    # blf.size(font_id, fsize, dpi)
     # height of one line
     mwidth, mheight = blf.dimensions(font_id, "Tp")  # uses high/low letters
 
@@ -835,19 +814,15 @@ def draw_text(myobj, pos2d, display_text, rgba, fsize, align='L', text_rot=0.0):
     # -------------------
     for line in mylines:
         text_width, text_height = blf.dimensions(font_id, line)
-        if align == 'C':
-            newx = x_pos - text_width / 2
-        elif align == 'R':
+        if right is True:
             newx = x_pos - text_width - gap
         else:
             newx = x_pos
-            blf.enable(font_id, ROTATION)
-            blf.rotation(font_id, text_rot)
         # calculate new Y position
         new_y = y_pos + (mheight * idx)
         # Draw
         blf.position(font_id, newx, new_y, 0)
-        blf.color(font_id, rgba[0], rgba[1], rgba[2], rgba[3])
+        bgl.glColor4f(rgb[0], rgb[1], rgb[2], rgb[3])
         blf.draw(font_id, " " + line)
         # sub line
         idx -= 1
@@ -855,9 +830,6 @@ def draw_text(myobj, pos2d, display_text, rgba, fsize, align='L', text_rot=0.0):
         if maxwidth < text_width:
             maxwidth = text_width
 
-    if align == 'L':
-        blf.disable(font_id, ROTATION)
-
     return maxwidth, maxheight
 
 
@@ -865,16 +837,14 @@ def draw_text(myobj, pos2d, display_text, rgba, fsize, align='L', text_rot=0.0):
 # Draw an OpenGL line
 #
 # -------------------------------------------------------------
-def draw_line(v1, v2, rgba):
-    coords = [(v1[0], v1[1]), (v2[0], v2[1])]
-    batch = batch_for_shader(shader, 'LINES', {"pos": coords})
-
+def draw_line(v1, v2):
     # noinspection PyBroadException
     try:
         if v1 is not None and v2 is not None:
-            shader.bind()
-            shader.uniform_float("color", rgba)
-            batch.draw(shader)
+            bgl.glBegin(bgl.GL_LINES)
+            bgl.glVertex2f(*v1)
+            bgl.glVertex2f(*v2)
+            bgl.glEnd()
     except:
         pass
 
@@ -883,16 +853,15 @@ def draw_line(v1, v2, rgba):
 # Draw an OpenGL triangle
 #
 # -------------------------------------------------------------
-def draw_triangle(v1, v2, v3, rgba):
-    coords = [(v1[0], v1[1]), (v2[0], v2[1]), (v3[0], v3[1])]
-    batch = batch_for_shader(shader, 'TRIS', {"pos": coords})
-
+def draw_triangle(v1, v2, v3):
     # noinspection PyBroadException
     try:
         if v1 is not None and v2 is not None and v3 is not None:
-            shader.bind()
-            shader.uniform_float("color", rgba)
-            batch.draw(shader)
+            bgl.glBegin(bgl.GL_TRIANGLES)
+            bgl.glVertex2f(*v1)
+            bgl.glVertex2f(*v2)
+            bgl.glVertex2f(*v3)
+            bgl.glEnd()
     except:
         pass
 
@@ -901,9 +870,7 @@ def draw_triangle(v1, v2, v3, rgba):
 # Draw an Arrow
 #
 # -------------------------------------------------------------
-def draw_arrow(v1, v2, rgba, size=20, a_typ="1", b_typ="1"):
-    if v1 is None or v2 is None:
-        return
+def draw_arrow(v1, v2, size=20, a_typ="1", b_typ="1"):
 
     rad45 = radians(45)
     rad315 = radians(315)
@@ -945,19 +912,19 @@ def draw_arrow(v1, v2, rgba, size=20, a_typ="1", b_typ="1"):
 
     # Triangle o Lines
     if a_typ == "1" or a_typ == "3":
-        draw_line(v1, v1a, rgba)
-        draw_line(v1, v1b, rgba)
+        draw_line(v1, v1a)
+        draw_line(v1, v1b)
 
     if b_typ == "1" or b_typ == "3":
-        draw_line(v2, v2a, rgba)
-        draw_line(v2, v2b, rgba)
+        draw_line(v2, v2a)
+        draw_line(v2, v2b)
 
     if a_typ == "2":
-        draw_triangle(v1, v1a, v1b, rgba)
+        draw_triangle(v1, v1a, v1b)
     if b_typ == "2":
-        draw_triangle(v2, v2a, v2b, rgba)
+        draw_triangle(v2, v2a, v2b)
 
-    draw_line(v1, v2, rgba)
+    draw_line(v1, v2)
 
 
 # -------------------------------------------------------------
@@ -965,16 +932,16 @@ def draw_arrow(v1, v2, rgba, size=20, a_typ="1", b_typ="1"):
 #
 # v1, v2 are corners (bottom left / top right)
 # -------------------------------------------------------------
-def draw_rectangle(v1, v2, rgba):
+def draw_rectangle(v1, v2):
     # noinspection PyBroadException
     try:
         if v1 is not None and v2 is not None:
             v1b = (v2[0], v1[1])
             v2b = (v1[0], v2[1])
-            draw_line(v1, v1b, rgba)
-            draw_line(v1b, v2, rgba)
-            draw_line(v2, v2b, rgba)
-            draw_line(v2b, v1, rgba)
+            draw_line(v1, v1b)
+            draw_line(v1b, v2)
+            draw_line(v2, v2b)
+            draw_line(v2b, v1)
     except:
         pass
 
@@ -996,39 +963,6 @@ def format_point(mypoint, pr):
     return fmt
 
 
-# -------------------------------------------------------------
-# Draw object num for debug
-#
-# -------------------------------------------------------------
-# noinspection PyUnresolvedReferences,PyUnboundLocalVariable,PyUnusedLocal
-def draw_object(context, myobj, region, rv3d):
-    scene = bpy.context.scene
-    rgba = scene.measureit_debug_obj_color
-    fsize = scene.measureit_debug_font
-    precision = scene.measureit_debug_precision
-    # --------------------
-    # object Loop
-    # --------------------
-    objs = bpy.context.scene.objects
-    obidxs = list(range(len(bpy.context.scene.objects)))
-    for o in obidxs:
-        # Display only selected
-        if scene.measureit_debug_select is True:
-            if objs[o].select_get() is False:
-                continue
-        a_p1 = Vector(get_location(objs[o]))
-        # Text
-        txt = ''
-        if scene.measureit_debug_objects is True:
-            txt += str(o)
-        if scene.measureit_debug_object_loc is True:
-            txt += format_point(a_p1, precision)
-        # converting to screen coordinates
-        txtpoint2d = get_2d_point(region, rv3d, a_p1)
-        draw_text(myobj, txtpoint2d, txt, rgba, fsize)
-    return
-
-
 # -------------------------------------------------------------
 # Draw vertex num for debug
 #
@@ -1040,17 +974,12 @@ def draw_vertices(context, myobj, region, rv3d):
         return
 
     scene = bpy.context.scene
-    rgba = scene.measureit_debug_vert_color
+    rgb = scene.measureit_debug_color
     fsize = scene.measureit_debug_font
     precision = scene.measureit_debug_precision
     # --------------------
     # vertex Loop
     # --------------------
-    if scene.measureit_debug_vert_loc_toggle == '1':
-        co_mult = lambda c: c
-    else:  # if global, convert local c to global
-        co_mult = lambda c: myobj.matrix_world @ c
-
     if myobj.mode == 'EDIT':
         bm = from_edit_mesh(myobj.data)
         obverts = bm.verts
@@ -1063,67 +992,24 @@ def draw_vertices(context, myobj, region, rv3d):
             if v.select is False:
                 continue
         # noinspection PyBroadException
-        # try:
-        a_p1 = get_point(v.co, myobj)
-        # converting to screen coordinates
-        txtpoint2d = get_2d_point(region, rv3d, a_p1)
-        # Text
-        txt = ''
-        if scene.measureit_debug_vertices is True:
-            txt += str(v.index)
-        if scene.measureit_debug_vert_loc is True:
-            txt += format_point(co_mult(v.co), precision)
-        draw_text(myobj, txtpoint2d, txt, rgba, fsize)
-        # except:
-        #     print("Unexpected error:" + str(exc_info()))
-        #     pass
+        try:
+            a_p1 = get_point(v.co, myobj)
+            # colour
+            bgl.glColor4f(rgb[0], rgb[1], rgb[2], rgb[3])
+            # converting to screen coordinates
+            txtpoint2d = get_2d_point(region, rv3d, a_p1)
+            # Text
+            txt = str(v.index)
+            if scene.measureit_debug_location is True:
+                txt += format_point(v.co, precision)
+            draw_text(myobj, txtpoint2d[0], txtpoint2d[1], txt, rgb, fsize)
+        except:
+            print("Unexpected error:" + str(exc_info()))
+            pass
 
     return
 
 
-# -------------------------------------------------------------
-# Draw edge num for debug
-#
-# -------------------------------------------------------------
-# noinspection PyUnresolvedReferences,PyUnboundLocalVariable,PyUnusedLocal
-def draw_edges(context, myobj, region, rv3d):
-    # Only meshes
-    if myobj.type != "MESH":
-        return
-
-    scene = bpy.context.scene
-    rgba = scene.measureit_debug_edge_color
-    fsize = scene.measureit_debug_font
-    precision = scene.measureit_debug_precision
-    # --------------------
-    # edge Loop
-    #
-    # uses lambda for edge midpoint finder (midf) because edit mode
-    # edge vert coordinate is not stored in same places as in obj mode
-    # --------------------
-    if myobj.mode == 'EDIT':
-        bm = from_edit_mesh(myobj.data)
-        obedges = bm.edges
-        obverts = None  # dummy value to avoid duplicating for loop
-        midf = lambda e, v: e.verts[0].co.lerp(e.verts[1].co, 0.5)
-    else:
-        obedges = myobj.data.edges
-        obverts = myobj.data.vertices
-        midf = lambda e, v: v[e.vertices[0]].co.lerp(v[e.vertices[1]].co, 0.5)
-
-    for e in obedges:
-        # Display only selected
-        if scene.measureit_debug_select is True:
-            if e.select is False:
-                continue
-        a_mp = midf(e, obverts)
-        a_p1 = get_point(a_mp, myobj)
-        # converting to screen coordinates
-        txtpoint2d = get_2d_point(region, rv3d, a_p1)
-        draw_text(myobj, txtpoint2d, str(e.index), rgba, fsize)
-    return
-
-
 # -------------------------------------------------------------
 # Draw face num for debug
 #
@@ -1135,8 +1021,8 @@ def draw_faces(context, myobj, region, rv3d):
         return
 
     scene = bpy.context.scene
-    rgba = scene.measureit_debug_face_color
-    rgba2 = scene.measureit_debug_norm_color
+    rgb = scene.measureit_debug_color2
+    rgb2 = scene.measureit_debug_color3
     fsize = scene.measureit_debug_font
     ln = scene.measureit_debug_normal_size
     th = scene.measureit_debug_width
@@ -1167,19 +1053,21 @@ def draw_faces(context, myobj, region, rv3d):
                 a_p1 = get_point(f.center, myobj)
 
             a_p2 = (a_p1[0] + normal[0] * ln, a_p1[1] + normal[1] * ln, a_p1[2] + normal[2] * ln)
-            # line setup
+            # colour + line setup
             bgl.glEnable(bgl.GL_BLEND)
             bgl.glLineWidth(th)
+            bgl.glColor4f(rgb[0], rgb[1], rgb[2], rgb[3])
             # converting to screen coordinates
             txtpoint2d = get_2d_point(region, rv3d, a_p1)
             point2 = get_2d_point(region, rv3d, a_p2)
             # Text
             if scene.measureit_debug_faces is True:
-                draw_text(myobj, txtpoint2d, str(f.index), rgba, fsize)
+                draw_text(myobj, txtpoint2d[0], txtpoint2d[1], str(f.index), rgb, fsize)
             # Draw Normal
             if scene.measureit_debug_normals is True:
                 bgl.glEnable(bgl.GL_BLEND)
-                draw_arrow(txtpoint2d, point2, rgba, 10, "99", "1")
+                bgl.glColor4f(rgb2[0], rgb2[1], rgb2[2], rgb2[3])
+                draw_arrow(txtpoint2d, point2, 10, "99", "1")
 
                 if len(obverts) > 2 and scene.measureit_debug_normal_details is True:
                     if myobj.mode == 'EDIT':
@@ -1199,11 +1087,11 @@ def draw_faces(context, myobj, region, rv3d):
                     b2d = get_2d_point(region, rv3d, a_p2)
                     c2d = get_2d_point(region, rv3d, a_p3)
                     # draw vectors
-                    draw_arrow(a2d, b2d, rgba, 10, "99", "1")
-                    draw_arrow(b2d, c2d, rgba, 10, "99", "1")
+                    draw_arrow(a2d, b2d, 10, "99", "1")
+                    draw_arrow(b2d, c2d, 10, "99", "1")
                     # Normal vector data
                     txt = format_point(normal, precision)
-                    draw_text(myobj, point2, txt, rgba2, fsize)
+                    draw_text(myobj, point2[0], point2[1], txt, rgb2, fsize)
 
         except:
             print("Unexpected error:" + str(exc_info()))
@@ -1270,7 +1158,7 @@ def get_point(v1, mainobject):
     # Using World Matrix
     vt = Vector((v1[0], v1[1], v1[2], 1))
     m4 = mainobject.matrix_world
-    vt2 = m4 @ vt
+    vt2 = m4 * vt
     v2 = [vt2[0], vt2[1], vt2[2]]
 
     return v2
@@ -1430,7 +1318,7 @@ def format_distance(fmt, units, value, factor=1):
         d_cm = value * (100 ** factor)
         tx_dist = fmt % d_cm
     # ------------------------
-    # Units millimeters
+    # Units milimiters
     # ------------------------
     elif units == "4":
         if hide_units is False:
@@ -1466,14 +1354,3 @@ def format_distance(fmt, units, value, factor=1):
         tx_dist = fmt % value
 
     return tx_dist
-
-
-# -------------------------------------------------------------
-# Get radian float based on angle choice
-#
-# -------------------------------------------------------------
-def get_angle_in_rad(fangle):
-    if fangle == 0:
-        return 0.0
-    else:
-        return radians(fangle)
